// XFile.h: interface for the XFile class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_XFILE_H__BF50AAEB_41CA_4AE1_A190_CD5B82531E93__INCLUDED_)
#define AFX_XFILE_H__BF50AAEB_41CA_4AE1_A190_CD5B82531E93__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <D3d8.h>
#include <Dxfile.h>
#include <list>

//	GUIDs not defined by the DirectX RM headers

// {3d82ab43-62da-11cf-ab39-0020af71e433}
DEFINE_GUID(TID_D3DRMHeader,
0x3d82ab43, 0x62da, 0x11cf, 0xab, 0x39, 0x00, 0x20, 0xaf, 0x71, 0xe4, 0x33);

// {<10DD46A3-775B-11cf-8F52-0040333594A3>}
DEFINE_GUID(TID_D3DRMQuaternion,
0x10DD46A3, 0x775B, 0x11cf, 0x8F, 0x52, 0x00, 0x40, 0x33, 0x35, 0x94, 0xA3);

// {<A3EB5D44-FC22-429D-9AFB-3221CB9719A6>}
DEFINE_GUID(TID_D3DRMPatch,
0xA3EB5D44, 0xFC22, 0x429D, 0x9A, 0xFB, 0x32, 0x21, 0xCB, 0x97, 0x19, 0xA6);

// {<D02C95CC-EDBA-4305-9B5D-1820D7704BBF>}
DEFINE_GUID(TID_D3DRMPatchMesh,
0xD02C95CC, 0xEDBA, 0x4305, 0x9B, 0x5D, 0x18, 0x20, 0xD7, 0x70, 0x4B, 0xBF);

// {<B8D65549-D7C9-4995-89CF-53A9A8B031E3>}
DEFINE_GUID(TID_D3DRMVertexDuplicationIndices,
0xB8D65549, 0xD7C9, 0x4995, 0x89, 0xCF, 0x53, 0xA9, 0xA8, 0xB0, 0x31, 0xE3);

// {<3CF169CE-FF7C-44ab-93C0-F78F62D172E2>}
DEFINE_GUID(TID_D3DRMXSkinMeshHeader,
0x3CF169CE, 0xFF7C, 0x44ab, 0x93, 0xC0, 0xF7, 0x8F, 0x62, 0xD1, 0x72, 0xE2);

// {<6F0D123B-BAD2-4167-A0D0-80224F25FABB>}
DEFINE_GUID(TID_D3DRMSkinWeights,
0x6F0D123B, 0xBAD2, 0x4167, 0xA0, 0xD0, 0x80, 0x22, 0x4F, 0x25, 0xFA, 0xBB);

struct __declspec(uuid("3d82ab45-62da-11cf-ab39-0020af71e433")) IDirectXFileDataReference;
struct __declspec(uuid("3d82ab44-62da-11cf-ab39-0020af71e433")) IDirectXFileData;

//	You will need to replace these with some other auto COM ptr
//	the ATL one sould work
typedef CD3DInterfacePtr<IDirectXFile>				IDirectXFilePtr;
typedef CD3DInterfacePtr<IDirectXFileEnumObject>	IDirectXFileEnumObjectPtr;
typedef CD3DInterfacePtr<IDirectXFileData>			IDirectXFileDataPtr;
typedef CD3DInterfacePtr<IDirectXFileObject>		IDirectXFileObjectPtr;
typedef CD3DInterfacePtr<IDirectXFileDataReference>	IDirectXFileDataReferencePtr;
typedef CD3DInterfacePtr<IDirectXFileBinary>		IDirectXFileBinaryPtr;

class XFile
{
	typedef std::list<IDirectXFileDataPtr>	flie_data_stack;

	IDirectXFilePtr				m_pXFile;
	IDirectXFileEnumObjectPtr	m_pEnumObject;
	IDirectXFileDataPtr			m_pCurrentObject;
	flie_data_stack				m_stackFileData;
	const GUID					*m_pCurrentType;

public:
	struct coords2d
	{
		float u;
		float v;
	};
	struct meshface
	{
		DWORD	nFaceVertexIndices;
		DWORD	pFaceVertexIndices[3];
	};

	XFile();
	virtual ~XFile();
	void Create(void *pvSource, const void *pvTemplate = NULL,
			DWORD dwTemplateSize = 0, DWORD dwLoadFlags = DXFILELOAD_FROMFILE,
			bool bLoadTemplatesFromFile = false,
			bool bUseStandardTemplates = true);
//	iteration
	bool Push();
	bool Pop();
	bool NextSibling();
	bool NextRoot();

//	identification
	bool IsCurrent(REFGUID refGuid);
	bool CurrentType(GUID &refGuid);
	bool CurrentID(GUID &refGuid);
	bool CurrentName(std::string &strName);

//	data retrieval
	bool GetString(LPCSTR szName, char *&pszName);
	bool GetStringArray(LPCSTR szName, char **&ppszName);
	bool GetDword(LPCSTR szName, DWORD &dwData);
	bool GetDwordArray(LPCSTR szName, DWORD *&pDWArray);
	bool GetFloat(LPCSTR szName, float &fData);
	bool GetFloatArray(LPCSTR szName, float *&pFArray);
	bool GetVector3(LPCSTR szName, D3DVECTOR &vecVector3);
	bool GetVector3Array(LPCSTR szName, D3DVECTOR *&pVectors);
	bool GetColorRGBA(LPCSTR szName, D3DCOLORVALUE &rgbColorRGBA);
	bool GetFaceArray(LPCSTR szName, meshface *&pFaces);
	bool Get2DTextureCoordArray(LPCSTR szName, coords2d *&pCoords);

//	utility
	IDirectXFileDataPtr& CurrentDataObject(void);

protected:
	bool GetChildDataObject(IDirectXFileDataPtr &pDataObject,
							const IDirectXFileDataPtr & pFileObject);

};

#endif //!defined(AFX_XFILE_H__BF50AAEB_41CA_4AE1_A190_CD5B82531E93__INCLUDED_)
